@@ -39,6 +39,9 @@ Testing Dependencies
 Usage
 =====
 
+OOPS Configuration
+++++++++++++++++++
+
 * Setup your configuration::
 
   >>> from oops_twisted import Config
@@ -55,6 +58,9 @@ be silently discarded.
 
  A helper 'defer_publisher' is supplied to do this for your convenience.
 
+Catching log.err calls
+++++++++++++++++++++++
+
 * create an OOPS log observer::
 
  >>> from oops_twisted import OOPSObserver
@@ -92,6 +98,37 @@ You can directly create an OOPS if you have a twisted failure object::
  >>> from twisted.python.failure import Failure
  >>> report = config.create(dict(twisted_failure=Failure()))
  >>> config.publish(report)
+
+Extending WSGI
+++++++++++++++
+
+oops_twisted supports an extended WSGI contract where if the returned iterator
+for the body implements t.w.i.IBodyProducer, then the iterator that
+oops_twisted's WSGI wrapper returns will also implement IBodyProducer. This is
+useful with a customised Twisted WSGI resource that runs IBodyProducer
+iterators in the IO loop, rather than using up a threadpool thread. To use this
+pass tracker=oops_twisted.wsgi.body_producer_tracker when calling
+oops_wsgi.make_app. Note that a non-twisted OOPS Config is assumed because 
+the WSGI protocol is synchronous: be sure to provide the oops_wsgi make_app
+with a non-twisted OOPS Config.
+
+If you are publishing with native OOPS publishers you may want to write a small
+synchronous publish-to-an-internal queue as you cannot use
+t.i.t.blockingCallFromThread: the WSGI protocol permits start_response to be
+called quite late, which may happen after an IBodyProducer has been returned to
+the WSGI gateway and all further code will be executing in the reactor thread.
+Specifically the call to startProducing may trigger start_response being called
+before the first write() occurs; and the call to start_response may trigger an
+OOPS being published if:
+ - it contains an exc_info value
+ - it has a status code matching the oops-on-status code values
+ - (in future) the response takes too long to start flowing
+Another route to exceptions is in startProducing itself, which may error, with
+similar consequences in that the oops config must be called into, and it has to
+be compatible with the config for start_response handling. If there is a need
+to address this, oops_twisted could take responsibility for exception handling
+in the IBodyProducer code path, with the cost of needing a second OOPS config
+- a native Twisted one.
 """
 
 
@@ -106,7 +143,7 @@ You can directly create an OOPS if you have a twisted failure object::
 # established at this point, and setup.py will use a version of next-$(revno).
 # If the releaselevel is 'final', then the tarball will be major.minor.micro.
 # Otherwise it is major.minor.micro~$(revno).
-__version__ = (0, 0, 4, 'beta', 0)
+__version__ = (0, 0, 5, 'beta', 0)
 
 __all__ = [
     'Config',
